"{ Package: 'stx:libscm/common' }"!

!ChangeSet methodsFor:'utilities'!

condenseChangesForPackageAfterCommit:aPackageSymbol
    "remove more changes for aPackageSymbol
     This is invoked when a project is checked into the repository."

    |changesToRemove|

    changesToRemove := self select:[:aChange |
        |removeThis mClass mthd|

        removeThis := false.
        (aChange isMethodChange or:[aChange isMethodRemoveChange]) ifTrue:[
            mClass := aChange changeClass.
            mClass notNil ifTrue:[
                mthd := mClass compiledMethodAt:(aChange selector).
                mthd isNil ifTrue:[
                    removeThis := (mClass package = aPackageSymbol)
                ] ifFalse:[
                    removeThis := (mthd package = aPackageSymbol)
                ]
            ].
        ] ifFalse:[
            (aChange isClassChange) ifTrue:[
                (aChange changeClass notNil) ifTrue:[
                    removeThis := (aChange changeClass package = aPackageSymbol)
                ].
            ].
        ].
        removeThis
    ].

    self condenseChanges:changesToRemove

    "Created: / 21-11-2012 / 01:45:08 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!Class methodsFor:'accessing'!

copyrightMethodSource
    "Return the source of the copyright method. If this class have
     no copyright method, then an initial one is generated using
     class definition's copyright method as template. If not even
     class definition has one then default empty one is returned.
    "

    | pkgdef |

    (self class includesSelector: #copyright) ifTrue:[
        ^(self class compiledMethodAt: #copyright) source.
    ].

    "/ OK, no copyright yet (new / old class?). Take project definition's
    "/ one and filter out all copyright lines...
    pkgdef := ProjectDefinition definitionClassForPackage: self package.
    (pkgdef notNil and:[pkgdef ~~ self]) ifTrue:[ 
        | source |

        source := pkgdef copyrightMethodSource.
        source notEmpty ifTrue:[ 
            | copyrights |

            source := source asStringCollection.
            copyrights := SCMCopyrightLine collectCopyrightsFromText: source.
            copyrights notEmpty ifTrue:[
                copyrights first reverseDo:[:copyright|
                    source removeIndex: copyright line.  
                ].
            ].
            source := source asString.
        ].
        ^ source
    ].

    "/ OK, no project definition or no copyright method there,
    "/ return nil
    ^ 'copyright
"
"
'

    "
    Class copyrightMethodSource
    SCMCopyrightUpdater copyrightMethodSource
    "

    "Created: / 27-05-2020 / 11:45:29 / Jan Vrany <jan.vrany@labware.com>"
    "Modified: / 27-05-2020 / 12:55:10 / Jan Vrany <jan.vrany@labware.com>"
! !

!UserPreferences methodsFor:'accessing-prefs-changes & history'!

historyManagerCopyrightHolder
    ^ self at: #historyManagerCopyrightHolder 
        ifAbsent:[(self historyManagerSignature upTo: $<) trimSeparators]

    "Created: / 27-05-2020 / 13:15:51 / Jan Vrany <jan.vrany@labware.com>"
! !

!UserPreferences methodsFor:'accessing-prefs-changes & history'!

historyManagerCopyrightHolder: aString
    aString isNil ifTrue:[ 
        self removeKey: #historyManagerCopyrightHolder
    ] ifFalse:[ 
        self at: #historyManagerCopyrightHolder put: aString
    ]

    "
    UserPreferences current historyManagerCopyrightHolder
    UserPreferences current historyManagerCopyrightHolder: 'LabWare'
    UserPreferences current historyManagerCopyrightHolder: nil
    "

    "Created: / 27-05-2020 / 13:16:41 / Jan Vrany <jan.vrany@labware.com>"
! !

!stx_libscm_common class methodsFor:'documentation'!

extensionsVersion_HG

    ^ '$Changeset: <not expanded> $'
! !
